Attribute VB_Name = "activeVB_AV"
Option Explicit

Private Declare Sub API_CopyByRef _
    Lib "kernel32" Alias "RtlMoveMemory" _
   (ByRef hpvDest As Any, _
    ByRef hpvSource As Any, _
    ByVal cbCopy As Long)

' CryptAV   Encryption/Decryption
'           Compatible to activeVB: "Advance Vigenre"
'
' CALL:     CryptAV(src, @tgt, pwd, cmd)
'
' IN:       bar:src Sourcedata
'           bar:tgt Targetdata
'           chr:pwd Password
'           bol:cmd .F.: Encrypt
'                   .T.: Decrypt
'
' OUT:      bol     success
'
Public Function CryptAV( _
    src() As Byte, _
    tgt() As Byte, _
    pwd As String, _
    cmd As Boolean) As Boolean
    
    Dim asm         As asmCryption
    Dim T1          As String       ' temp
    Dim I1          As Long         ' loop counter
    Dim P1          As Long         ' act src-position
    Dim P2          As Long         ' act tgt-position
    Dim sha()       As Long         ' hash (random base)
    Dim tmp(8)      As Long         ' temp
    Dim bar()       As Byte         ' converting long to string
    Dim chk         As Boolean      ' password valid?
    
    
    Set asm = New asmCryption
    
    ' create a hash from password/get timestamp
    sha = asm.Hash(StrConv(pwd, vbFromUnicode))
    ReDim Preserve sha(8)
    sha(8) = DateDiff("s", DateSerial(1970, 1, 1), Now)
    ReDim tgt(UBound(src) + IIf(cmd, -36, 36))
    
    If cmd Then
        ' onDecode
        If UBound(src) >= 36 Then
            API_CopyByRef tmp(0), src(0), 36
            chk = lngCMP(sha(0), tmp(0), 7) ' compare hash
            sha(8) = tmp(8)      ' get timestamp from file
            P1 = 36              ' set first src-position
        End If
        
    Else
        ' onEncode
        API_CopyByRef tgt(0), sha(0), 36 ' copy hash to tgt
        P2 = 36                    ' set first tgt-position
    End If
    
    If cmd Imp chk Then
        ' create hash from pwd+timestamp
        bar = LNG2CHR(sha(8))
        bar(1) = 0: bar(3) = 0: bar(5) = 0: bar(7) = 0
        T1 = Left$(bar, 2) & pwd & vbNullChar & Mid$(bar, 3)
        sha = asm.Hash(StrConv(T1, vbFromUnicode))
        
        ' loop
        If chk Then lngROR sha(0), 128, 7, 3
        
        For I1 = P1 To UBound(src)
            tgt(P2) = src(I1) Xor (lngRND(sha(I1 And 7)) And 255)
            P2 = P2 + 1
        Next
    
        CryptAV = True
    End If
End Function

' create standardized long-array from message
'
Private Function x_MSG2LNG( _
    src() As Byte, _
    Optional AddBytes As Long = 0, _
    Optional BigEndian As Boolean = True) As Long()
    
    Dim L1    As Long
    Dim N1    As Long
    Dim msg() As Long
    
    ' calculate number of blocks/redim msg acc.
    L1 = UBound(src) + 1
    N1 = ((L1 + 9 + AddBytes + 63) And -64&) / 4 - 1
    ReDim msg(N1)
    If AddBytes < 0 Then L1 = L1 + AddBytes
    
    ' copy message+80h to long array
    API_CopyByRef msg(0), src(0), L1
    API_CopyByRef ByVal VarPtr(msg(0)) + L1, &H80&, 1
    If BigEndian Then lngSWP msg(0), msg(0), N1
    
    ' set message length/bits
    msg(N1) = L1
    lngROL msg(N1 - 1), 3, 1, 3
    
    x_MSG2LNG = msg
End Function

