Attribute VB_Name = "cpuClock"
'---------------------------------------------------------------------------------------
' CPU-clock zum Messen kleiner Zeiten nutzen
' (bentigt einen Pentium-Prozessor!)
'
' (softKUS) - X/2003
'---------------------------------------------------------------------------------------

Option Explicit

Dim cur_sec     As Currency
Dim cur_dmy     As Currency
Dim cur_txt     As String

' Aufruf von asm-Funktionen (Deklaration fr Longs)
Private Declare Function ASM_cdLong _
    Lib "user32" _
    Alias "CallWindowProcA" _
    (ByRef adr As Long, _
     ByVal PA1 As Long, _
     ByVal PA2 As Long, _
     ByVal PA3 As Long, _
     ByVal PA4 As Long) As Long

Private Declare Function API_PCounter _
    Lib "kernel32" _
    Alias "QueryPerformanceCounter" _
    (lpPerformanceCount As Currency) As Long
        
Private Declare Function API_PFrequency _
    Lib "kernel32" _
    Alias "QueryPerformanceFrequency" _
    (lpFrequency As Currency) As Long

' clkFMT        Formatieren von Zeispannen
'
' AUFRUF:       clkFMT(cur, [cnt])
'
' EIN:          cur:cur Zu formatierender Wert
'               lng:cnt Anzahl Nachkommastellen (Vorgabe=2)
'
' AUS:          chr     Formatierter Text
'
Function clkFMT( _
    cur As Currency, _
    Optional cnt As Long = 2) As String
    
    Dim sec As Double
    Dim txt As String
    
    txt = "0." & String(cnt, "0")
    
    If cur < 0 Then
        clkFMT = cur_txt
        
    ElseIf cur <> 0 Then
        If cur_sec Then sec = cur / cur_sec
        Select Case Log(sec) / Log(10)
        Case Is > 1
            clkFMT = Format(sec / 60, txt) & " m"
            
        Case Is > -1
            clkFMT = Format(sec, txt) & " s"
            
        Case Is > -4
            clkFMT = Format(sec * 1000, txt) & " ms"
            
        Case Else
            clkFMT = Format(sec * 1000000, txt) & " s"
        End Select
    End If
End Function

' clkGET        Einlesen des CPU-internen Taktzhlers
'
' AUFRUF:       clkGET(cur(), [NR])
'
' EIN:          cur:cur()   Array von mind. 2 Currency-Werten
'               lng:NR      Zeiger auf cur()-Element (Vorgabe=0)
'
' AUS:          cur(NR)     Aktueller CPU-Taktzhler
'               cur(NR+1)   Differenz zw. aktuellem cur(NR) und
'                           cur(NR) beim Funktionsaufruf
'
' HINWEIS:      Um Zeiten zu messen, sollte clkGET einmal zur Initiali-
'               sierung aufgerufen werden (setzt cur(NR)) und ein weiteres
'               mal zur Berechnung der Zeitspanne (setzt cur(NR+1))
'
Function clkGET( _
    cur() As Currency, _
    Optional NR As Long) As Currency
    
    Static asm(9) As Long

    If asm(0) = 0 Then
        asm(0) = &H4C8B310F:  asm(1) = &H31FF0424
        asm(2) = &H890471FF:  asm(3) = &H4518901
        asm(4) = &H424442B:   asm(5) = &H8924141B
        asm(6) = &H51890841:  asm(7) = &HF95A580C
        asm(8) = &H10C2C01B:  asm(9) = &H0
    End If
    
    ' *****************************************************
    
    On Error Resume Next ' Fehler: ungltiges Array abfangen
    
    If UBound(cur) >= NR + 1 Then
        ASM_cdLong asm(0), VarPtr(cur(NR)), 0, 0, 0
        clkGET = cur(NR + 1)
    End If
End Function

' clkINI        Initialisieren
'
' AUFRUF:       clkINI([vl])
'
' EIN:          dbl:vl  CheckRate (Vorgabe = 0.5)
'                       Erklrung s. unten
'
' AUS:          cur     Taktfrequenz
'
' setzt:        cur_sec Anzahl Takte/Sekunde
'               cur_txt Taktfrequenz als formatierter Text
'               cur_dmy Durchschnittliche Anzahl Takte, die
'                       fr clkGET() bentigt werden
'
' HINWEIS:      CheckRate gibt in 1/sec die Zeitspanne an,
'               die clkINI zwischen dem Lesen des CPU-Taktzhlers
'               verstreichen lt. Je hher der Wert, desto genauer
'               ist das Ergebnis (cur_sec/dur_dmy)
'
Function clkINI(Optional chkRate As Double = 0.5) As Currency
    Dim cur(3) As Currency
    Dim cu1    As Currency
    Dim cu2    As Currency
    
    If cur_sec = 0 Then
        API_PFrequency cu1
        API_PCounter cu2
        cu1 = cu2 + cu1 * chkRate
        
        clkGET cur, 0
        clkGET cur, 2
        
        Do: API_PCounter cu2
            cur_dmy = (cur_dmy + clkGET(cur, 2)) / 2
        Loop Until cu2 >= cu1
        
        cur_sec = clkGET(cur, 0) * (1 / chkRate)
        cu1 = IIf(cur_sec > 100000, 100000, 100)
        cur_txt = "Running at " & _
            Format(cur_sec / cu1, "0.00") & _
            IIf(cu1 = 100, " MHz", " GHz")
    End If
    
    clkINI = cur_sec
End Function

' clkWAIT       Sleep-Funktion
'
' AUFRUF:       clkWAIT(Zeit, [cnt])
'
' EIN:          dbl:Zeit    Abzuwartende Zeit/sec
'               cur:Cnt     Zhler
'
' AUS:          dbl         Tatschlich verstrichene Zeit/sec
'               cnt         Anzahl der Loops der ASM-Funktion
'                           > 1: Ergebnis ist verllich
'
' HINWEIS:      Die ASM-Funktion wird mit einem Array-Parameter
'               aufgerufen:
'               0: CPU-Taktzhler beim Funktionsaufruf
'               1: Differenz 0/1 und aktueller CPU-Taktzhler
'               2: Anzahl Takte, die abgewartet werden soll
'               3: Loop-Zhler. Ist der Loop-Zhler nach Ver-
'                  lassen der Funktion >1, wurde die ASM-Schleife
'                  mehr als einmal durchlaufen und ist das Ergeb-
'                  nis verllich. Allerdings wird der Aufwand,
'                  der hier fr den Funktionsaufruf betrieben wird,
'                  nicht bercksichtigt!
'
Function clkWAIT(sec As Double, Optional cnt As Currency) As Double
    Static asm(12) As Long

    If asm(0) = 0 Then
        asm(0) = &H4C8B310F:  asm(1) = &H1890424
        asm(2) = &H33045189:  asm(3) = &H184189C0
        asm(4) = &HF1C4189:   asm(5) = &H18418331
        asm(6) = &H1C518301:  asm(7) = &H1B012B00
        asm(8) = &H41890451:  asm(9) = &HC518908
        asm(10) = &H1B10412B: asm(11) = &HE3721451
        asm(12) = &H10C2
    End If
    
    ' *****************************************************

    Dim cur(3) As Currency
    
    cur(2) = sec * cur_sec
    ASM_cdLong asm(0), VarPtr(cur(0)), 0, 0, 0
    cnt = cur(3)
    clkWAIT = cur(1) / cur_sec
End Function
